"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_1 = require("graphql");
const graphql_2 = require("./graphql");
class Context {
    constructor(ast) {
        this.ast = ast;
        this.typesUsed = new Set();
        this.specialTypesUsed = new Set();
        this.importsUsed = new Map();
    }
    addUsedExternalFragments(fragments) {
        for (const fragmentName of fragments) {
            const fragment = this.ast.fragments[fragmentName];
            this.addUsedImport(`${fragmentName}Fragment`, fragment.filePath);
        }
    }
    addUsedImport(name, source) {
        const imports = this.importsUsed.get(source) || new Set();
        imports.add(name);
        this.importsUsed.set(source, imports);
    }
    addUsedSpecialType(type) {
        this.specialTypesUsed.add(type);
    }
    addUsedType(type) {
        const { typesUsed } = this;
        if (typesUsed.has(type)) {
            return;
        }
        if (type instanceof graphql_1.GraphQLEnumType) {
            typesUsed.add(type);
        }
        else if (type instanceof graphql_1.GraphQLInputObjectType) {
            typesUsed.add(type);
            const fields = type.getFields();
            Object
                .keys(fields)
                .map((fieldName) => fields[fieldName])
                .forEach((typeField) => this.addUsedType(typeField.type));
        }
        else if (type instanceof graphql_1.GraphQLScalarType && !graphql_2.builtInScalarMap.hasOwnProperty(type.name)) {
            typesUsed.add(type);
        }
        else if (type instanceof graphql_1.GraphQLNonNull || type instanceof graphql_1.GraphQLList) {
            this.addUsedType(type.ofType);
        }
    }
}
exports.default = Context;
